//
// Written by Sander van Rossen
//	You are free to use this piece of code any way you like
//
#include "settings.h"
#include "types.h"
#include "constants.h"
#include "convertbits.h"
#include "colordescription.h"
#include "converter.h"



//
// All color bit sizes must be byte aligned, meaning that you can either
// divide the size of the color by 8, or 8 can be divided by the 
// size of the color..
//
// In it's current form the total size of the color is limited to 32bit
//
// ----------------------------------------------------------------------
//						  color-type    R G B A  x
typedef ColorDescription< COLOR_ABGR ,  8,8,8,8, 0 	> TYPE_ABGR_32_8888;
typedef ColorDescription< COLOR_ARGB ,  8,8,8,8, 0	> TYPE_ARGB_32_8888;
typedef ColorDescription< COLOR_BGRA ,  8,8,8,8, 0	> TYPE_BGRA_32_8888;
typedef ColorDescription< COLOR_RGBA ,  8,8,8,8, 0	> TYPE_RGBA_32_8888;
typedef ColorDescription< COLOR_RGB  ,  8,8,8,0, 8	> TYPE_RGB_32_888;
typedef ColorDescription< COLOR_RGB  ,  8,8,8,0, 0	> TYPE_RGB_24_888;
typedef ColorDescription< COLOR_RGBA ,  4,4,4,4, 0	> TYPE_RGBA_16_4444;
typedef ColorDescription< COLOR_RGBA ,  5,5,5,1, 0	> TYPE_RGBA_16_5551;
typedef ColorDescription< COLOR_RGBA ,  5,5,5,0, 1	> TYPE_RGB_16_555;
typedef ColorDescription< COLOR_RGB  ,  5,6,5,0, 0	> TYPE_RGB_16_565;
typedef ColorDescription< COLOR_ALPHA,  0,0,0,8, 0	> TYPE_ALPHA_8;
typedef ColorDescription< COLOR_ALPHA,  0,0,0,4, 0	> TYPE_ALPHA_4;
typedef ColorDescription< COLOR_ALPHA,  0,0,0,2, 0	> TYPE_ALPHA_2;
typedef ColorDescription< COLOR_ALPHA,  0,0,0,1, 0	> TYPE_ALPHA_1;




#define src_test	TYPE_ARGB_32_8888
#define dst_test	TYPE_ALPHA_8


#define	BYTE_ALIGN(x) (((UInt)x + (BITS_PER_BYTE-1)) / BITS_PER_BYTE)


enum
{
	width		= 256,
	height		= 256,
	num_pixels	= width * height,
	src_size	= BYTE_ALIGN(num_pixels * src_test::COLOR_STORAGE_BITS),
	dst_size	= BYTE_ALIGN(num_pixels * dst_test::COLOR_STORAGE_BITS)

};

void main(void)
{
	UChar	input_buffer	[src_size];
	UChar	output_buffer	[dst_size];
	int		i;

	// clear output buffer
	for (i = 0;i < dst_size;i++) output_buffer[i]	= 0;

	int j = (src_size - (src_size%15));
	// fill input buffer with predictable junk
	for (i = 0;i < j;i+=16)
	{
		input_buffer[i +  0] = 0x01;
		input_buffer[i +  1] = 0x12;
		input_buffer[i +  2] = 0x23;
		input_buffer[i +  3] = 0x34;
		input_buffer[i +  4] = 0x45;
		input_buffer[i +  5] = 0x56;
		input_buffer[i +  6] = 0x67;
		input_buffer[i +  7] = 0x78;
		input_buffer[i +  8] = 0x89;
		input_buffer[i +  9] = 0x9A;
		input_buffer[i + 10] = 0xAB;
		input_buffer[i + 11] = 0xBC;
		input_buffer[i + 12] = 0xCD;
		input_buffer[i + 13] = 0xDE;
		input_buffer[i + 14] = 0xEF;
		input_buffer[i + 15] = 0xF0;
	}
	for (i=j;i < src_size;i++) input_buffer[i] = 0xFF;
	
	// convert the colors
	ConvertColors<src_test, dst_test>::
		Convert(output_buffer, input_buffer, src_size);
}